@@ -288,9 +288,9 @@ empathy_main_window_show (void)
 	empathy_status_presets_get_all ();
 
 	list_iface = EMPATHY_CONTACT_LIST (empathy_contact_manager_new ());
-	empathy_contact_list_setup (list_iface);
 	window->list_store = empathy_contact_list_store_new (list_iface);
 	window->list_view = empathy_contact_list_view_new (window->list_store);
+	empathy_contact_list_view_set_interactive (window->list_view, TRUE);
 	g_object_unref (list_iface);
 
 	gtk_widget_show (GTK_WIDGET (window->list_view));
@@ -897,13 +897,13 @@ main_window_notify_compact_contact_list_cb (EmpathyConf        *conf,
 }
 
 static void
-main_window_notify_sort_criterium_cb (EmpathyConf        *conf,
+main_window_notify_sort_criterium_cb (EmpathyConf       *conf,
 				      const gchar       *key,
 				      EmpathyMainWindow *window)
 {
 	gchar *str = NULL;
 
-	if (empathy_conf_get_string (conf, key, &str)) {
+	if (empathy_conf_get_string (conf, key, &str) && str) {
 		GType       type;
 		GEnumClass *enum_class;
 		GEnumValue *enum_value;
@@ -915,7 +915,7 @@ main_window_notify_sort_criterium_cb (EmpathyConf        *conf,
 
 		if (enum_value) {
 			empathy_contact_list_store_set_sort_criterium (window->list_store, 
-								      enum_value->value);
+								       enum_value->value);
 		}
 	}
 }
