@@ -230,104 +230,6 @@ empathy_pixbuf_from_icon_name (const gchar *icon_name,
 	return pixbuf;
 }
 
-GdkPixbuf *
-empathy_pixbuf_from_smiley (EmpathySmiley type,
-			   GtkIconSize  icon_size)
-{
-	const gchar *icon_id;
-
-	switch (type) {
-	case EMPATHY_SMILEY_NORMAL:       /*  :)   */
-		icon_id = "stock_smiley-1";
-		break;
-	case EMPATHY_SMILEY_WINK:         /*  ;)   */
-		icon_id = "stock_smiley-3";
-		break;
-	case EMPATHY_SMILEY_BIGEYE:       /*  =)   */
-		icon_id = "stock_smiley-2";
-		break;
-	case EMPATHY_SMILEY_NOSE:         /*  :-)  */
-		icon_id = "stock_smiley-7";
-		break;
-	case EMPATHY_SMILEY_CRY:          /*  :'(  */
-		icon_id = "stock_smiley-11";
-		break;
-	case EMPATHY_SMILEY_SAD:          /*  :(   */
-		icon_id = "stock_smiley-4";
-		break;
-	case EMPATHY_SMILEY_SCEPTICAL:    /*  :/   */
-		icon_id = "stock_smiley-9";
-		break;
-	case EMPATHY_SMILEY_BIGSMILE:     /*  :D   */
-		icon_id = "stock_smiley-6";
-		break;
-	case EMPATHY_SMILEY_INDIFFERENT:  /*  :|   */
-		icon_id = "stock_smiley-8";
-		break;
-	case EMPATHY_SMILEY_TOUNGE:       /*  :p   */
-		icon_id = "stock_smiley-10";
-		break;
-	case EMPATHY_SMILEY_SHOCKED:      /*  :o   */
-		icon_id = "stock_smiley-5";
-		break;
-	case EMPATHY_SMILEY_COOL:         /*  8)   */
-		icon_id = "stock_smiley-15";
-		break;
-	case EMPATHY_SMILEY_SORRY:        /*  *|   */
-		icon_id = "stock_smiley-12";
-		break;
-	case EMPATHY_SMILEY_KISS:         /*  :*   */
-		icon_id = "stock_smiley-13";
-		break;
-	case EMPATHY_SMILEY_SHUTUP:       /*  :#   */
-		icon_id = "stock_smiley-14";
-		break;
-	case EMPATHY_SMILEY_YAWN:         /*  |O   */
-		icon_id = "";
-		break;
-	case EMPATHY_SMILEY_CONFUSED:     /*  :$   */
-		icon_id = "stock_smiley-17";
-		break;
-	case EMPATHY_SMILEY_ANGEL:        /*  O)   */
-		icon_id = "stock_smiley-18";
-		break;
-	case EMPATHY_SMILEY_OOOH:         /*  :x   */
-		icon_id = "stock_smiley-19";
-		break;
-	case EMPATHY_SMILEY_LOOKAWAY:     /*  *)   */
-		icon_id = "stock_smiley-20";
-		break;
-	case EMPATHY_SMILEY_BLUSH:        /*  *S   */
-		icon_id = "stock_smiley-23";
-		break;
-	case EMPATHY_SMILEY_COOLBIGSMILE: /*  8D   */
-		icon_id = "stock_smiley-25";
-		break;
-	case EMPATHY_SMILEY_ANGRY:        /*  :@   */
-		icon_id = "stock_smiley-16";
-		break;
-	case EMPATHY_SMILEY_BOSS:         /*  @)   */
-		icon_id = "stock_smiley-21";
-		break;
-	case EMPATHY_SMILEY_MONKEY:       /*  #)   */
-		icon_id = "stock_smiley-22";
-		break;
-	case EMPATHY_SMILEY_SILLY:        /*  O)   */
-		icon_id = "stock_smiley-24";
-		break;
-	case EMPATHY_SMILEY_SICK:         /*  +o(  */
-		icon_id = "stock_smiley-26";
-		break;
-
-	default:
-		g_assert_not_reached ();
-		icon_id = NULL;
-	}
-
-
-	return empathy_pixbuf_from_icon_name (icon_id, icon_size);
-}
-
 const gchar *
 empathy_icon_name_from_account (McAccount *account)
 {
@@ -378,8 +280,7 @@ empathy_icon_name_for_presence (EmpathyPresence *presence)
 const gchar *
 empathy_icon_name_for_contact (EmpathyContact *contact)
 {
-	EmpathyPresence     *presence;
-	EmpathySubscription  subscription;
+	EmpathyPresence *presence;
 
 	g_return_val_if_fail (EMPATHY_IS_CONTACT (contact),
 			      EMPATHY_IMAGE_OFFLINE);
@@ -389,12 +290,7 @@ empathy_icon_name_for_contact (EmpathyContact *contact)
 		return empathy_icon_name_for_presence (presence);
 	}
 
-	subscription = empathy_contact_get_subscription (contact);
-	if (!(subscription & EMPATHY_SUBSCRIPTION_FROM)) {
-		return EMPATHY_IMAGE_PENDING;
-	}
-
-	return EMPATHY_IMAGE_OFFLINE;
+	return EMPATHY_IMAGE_UNKNOWN;
 }
 
 static void
